unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ShlObj, ComObj, ActiveX, StdCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    procedure Button1Click(Sender: TObject);
  private
    procedure GetDirs(RootDir: string; ADirList: TStrings);
    function IfFolderShared(FullFolderPath: string): Boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

function TForm1.IfFolderShared(FullFolderPath: string): Boolean;

  //Convert TStrRet to string
  function StrRetToString(PIDL: PItemIDList; StrRet: TStrRet; Flag: string = ''): string;
  var
    P: PChar;
  begin 
    case StrRet.uType of 
      STRRET_CSTR: 
        SetString(Result, StrRet.cStr, lStrLen(StrRet.cStr)); 
      STRRET_OFFSET: 
        begin 
          P := @PIDL.mkid.abID[StrRet.uOffset - SizeOf(PIDL.mkid.cb)]; 
          SetString(Result, P, PIDL.mkid.cb - StrRet.uOffset); 
        end; 
      STRRET_WSTR: 
        if Assigned(StrRet.pOleStr) then 
          Result := StrRet.pOleStr 
        else 
          Result := ''; 
    end; 
    { This is a hack bug fix to get around Windows Shell Controls returning 
      spurious "?"s in date/time detail fields } 
    if (Length(Result) > 1) and (Result[1] = '?') and (Result[2] in ['0'..'9']) then 
      Result := StringReplace(Result, '?', '', [rfReplaceAll]); 
  end; 

  //Get Desktop's IShellFolder interface 
  function DesktopShellFolder: IShellFolder; 
  begin 
    OleCheck(SHGetDesktopFolder(Result)); 
  end; 

  //delete the first ID from IDList 
  function NextPIDL(IDList: PItemIDList): PItemIDList;
  begin 
    Result := IDList; 
    Inc(PChar(Result), IDList^.mkid.cb); 
  end; 

  //get the length of IDList 
  function GetPIDLSize(IDList: PItemIDList): Integer; 
  begin 
    Result := 0; 
    if Assigned(IDList) then 
    begin 
      Result := SizeOf(IDList^.mkid.cb); 
      while IDList^.mkid.cb <> 0 do 
      begin 
        Result := Result + IDList^.mkid.cb; 
        IDList := NextPIDL(IDList); 
      end; 
    end; 
  end; 

  //get ID count from IDList 
  function GetItemCount(IDList: PItemIDList): Integer; 
  begin 
    Result := 0; 
    while IDList^.mkid.cb <> 0 do 
    begin 
      Inc(Result); 
      IDList := NextPIDL(IDList); 
    end;
  end; 

  //create an ItemIDList object 
  function CreatePIDL(Size: Integer): PItemIDList; 
  var 
    Malloc: IMalloc; 
  begin 
    OleCheck(SHGetMalloc(Malloc)); 

    Result := Malloc.Alloc(Size); 
    if Assigned(Result) then 
      FillChar(Result^, Size, 0); 
  end; 

  function CopyPIDL(IDList: PItemIDList): PItemIDList; 
  var 
    Size: Integer; 
  begin 
    Size   := GetPIDLSize(IDList); 
    Result := CreatePIDL(Size); 
    if Assigned(Result) then 
      CopyMemory(Result, IDList, Size); 
  end; 

  //get the last ItemID from AbsoluteID 
  function RelativeFromAbsolute(AbsoluteID: PItemIDList): PItemIDList; 
  begin 
    Result := AbsoluteID; 
    while GetItemCount(Result) > 1 do
      Result := NextPIDL(Result); 
    Result := CopyPIDL(Result); 
  end; 

  //remove the last ID from IDList 
  procedure StripLastID(IDList: PItemIDList); 
  var 
    MarkerID: PItemIDList; 
  begin 
    MarkerID := IDList; 
    if Assigned(IDList) then 
    begin 
      while IDList.mkid.cb <> 0 do
      begin 
        MarkerID := IDList; 
        IDList   := NextPIDL(IDList); 
      end; 
      MarkerID.mkid.cb := 0; 
    end; 
  end; 

  //if Flag include Element 
  function IsElement(Element, Flag: Integer): Boolean; 
  begin 
    Result := Element and Flag <> 0; 
  end; 
var 
  P: Pointer; 
  NumChars, Flags: LongWord;
  ID, NewPIDL, ParentPIDL: PItemIDList; 
  ParentShellFolder: IShellFolder; 
begin 
  Result := False; 
  NumChars := Length(FullFolderPath); 
  P  := StringToOleStr(FullFolderPath); 
  //get the folder's full ItemIDList 
  OleCheck(DesktopShellFolder.ParseDisplayName(0, nil, P, NumChars, NewPIDL, Flags)); 
  if NewPIDL <> nil then 
  begin
    ParentPIDL := CopyPIDL(NewPIDL); 
    StripLastID(ParentPIDL);      //get the folder's parent object's ItemIDList 

    ID := RelativeFromAbsolute(NewPIDL);  //get the folder's relative ItemIDList 

    //get the folder's parent object's IShellFolder interface 
    OleCheck(DesktopShellFolder.BindToObject(ParentPIDL, nil, IID_IShellFolder, 
      Pointer(ParentShellFolder))); 

    if ParentShellFolder <> nil then 
    begin 
      Flags := SFGAO_SHARE;
      //get the folder's attributes
      OleCheck(ParentShellFolder.GetAttributesOf(1, ID, Flags));
      if IsElement(SFGAO_SHARE, Flags) then Result := True;
    end;
  end;
end;


procedure TForm1.GetDirs(RootDir: string; ADirList: TStrings);
var search: TSearchRec;
begin
  if FindFirst(RootDir + '*.*', faDirectory, search) = 0 then
  begin
    repeat
      if ((search.Attr and faDirectory) = faDirectory) and
          (search.Name <> '.') and (search.Name <> '..') then begin
        ADirList.Add(RootDir + search.Name);
        GetDirs(RootDir + search.Name + '\', ADirList);
      end;
    until FindNext(search) <> 0;
    FindClose(search);
  end;
end;


procedure TForm1.Button1Click(Sender: TObject);
const
  Size: Integer = 200;
var
  pDrive: PChar;
  i, iLength: Integer;
  slDir: TStringList;
begin
  Screen.Cursor := crHourGlass;
  slDir := TStringList.Create;
  try
    GetMem(pDrive, Size);
    try
      iLength := GetLogicalDriveStrings(Size, pDrive);
      for i := 0 to iLength-1 do begin
        if (pDrive[i] >= 'A') and (pDrive[i] <= 'E') then begin
          // ˬdUϺЮڥؿL
          if IfFolderShared(pDrive[i] + ':\') then
            Memo1.Lines.Add(pDrive[i] + ':\');

          GetDirs(pDrive[i] + ':\', slDir);
        end;
      end;
    finally
      FreeMem(pDrive);
    end;

    for i := 0 to slDir.Count -1 do begin
      if IfFolderShared(slDir.Strings[i]) then
        Memo1.Lines.Add(slDir.Strings[i]);
    end;
  finally
    slDir.Free;
    Screen.Cursor := crDefault;
  end;
end;

end.
